/*
 * Copyright © 2006 - 2007 Claudio Corsi, Paolo Ferragina
 *
 * This file is part of SmallText library.
 *
 * SmallText is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * SmallText is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package it.unipi.di.textdb;

import it.unipi.di.util.Range;

import java.io.IOException;

/**
 * Search support for TextDBs. 
 * It includes the possibility to (prefix-)search for a string, or get its rank in 
 * the underlying TextDB. We assume that the records are lexicographically sorted,
 * so that it makes sense to say that records having an equal prefix 
 * are contiguous in the TextDB.
 * 
 * @author Claudio Corsi
 * @author Paolo Ferragina
 *
 */
public interface SearchableDB {

	/**
	 * Returns the position in this TextDB of the input string. 
	 * As the underlying TextDB is a sorted list of records
	 * (strings), if p is not found, the returned value is 
	 * the negative value of the position where the input 
	 * string should be placed in the TextDB.
	 * 
	 * @param p the string to be searched
	 * @return the position pos of s, if s occurs in the TextDB, or the value -pos
	 * @throws IOException
	 */
	public int rank(String p) throws IOException;
	
	/**
	 * Returns the range [i, j) of consecutive records in the TextDB
	 * that are prefixed by string p. Positions are counted from 1.
	 * 
	 * @param p the prefix to search
	 * @return the range [i, j) of records sharing the common prefix p
	 * @throws IOException
	 */
	public Range prefix(String p) throws IOException;
	
}
