/*
 * Copyright © 2006 - 2007 Claudio Corsi, Paolo Ferragina
 *
 * This file is part of SmallText library.
 *
 * SmallText is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * SmallText is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package it.unipi.di.tokenizer;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

/**
 * A {@link Tokenizer} for a list of URLs.
 * Each URL is tokenized into: protocol, host, port, directories of 
 * the path, query part. The token '\n' is added.
 * 
 * @author Claudio Corsi
 * @author Paolo Ferragina
 *
 */
public class URLTokenizer extends TermTokenizer {
	
	/**
	 * Create a new URLTokenizer over the given file.
	 * 
	 * @param file the file containing a list of URLs to split in tokens (separated by '\n')
	 * @throws IOException
	 */
	public URLTokenizer(String file) throws IOException {
		super(file);
	}

	protected String[] split(String line) {
		
		ArrayList<String> tokens = new ArrayList<String>();
		
		try {
			URL url = new URL(line);
			
			tokens.add(url.getProtocol() + "://");
			tokens.add(url.getHost());
			
			if (url.getPort() != -1) tokens.add(":" + url.getPort());
			
			String path = url.getPath();
			if (!path.equals("")) {
				StringBuffer sb = new StringBuffer();
				for (int i = 0; i < path.length(); i++) {
					
					char ch = path.charAt(i);
					
					if (ch == '/') {
						String part = sb.toString();
						if (part.length() != 0) tokens.add(part);
						sb.setLength(0);
						tokens.add("/");
					}
					else sb.append(ch);
				}
				String part = sb.toString();
				if (part.length() != 0) tokens.add(part);
			}
			
			if (url.getQuery() != null) tokens.add("?" + url.getQuery());
		}
		catch(MalformedURLException mex) {
			System.err.println(" >> WARNING: malformed URL " + line + ": " + mex.getMessage());
			tokens.add(line);
		}

		tokens.add("\n");	// add "\n" as term
		
		return (String[])tokens.toArray(new String[0]);
	}
}
